#include<stdlib.h>
#include<stdio.h>
#include<errno.h>

#define NO_INST	5

#define ERR_USAGE	1
#define ERR_CREATE_FILE	2
#define ERR_WRITE	3

// Swap 2 byte, 16 bit values
#define Swap2Bytes(val) \
 ( (((val) >> 8) & 0x00FF) | (((val) << 8) & 0xFF00) )

// Swap 4 byte, 32 bit values:

#define Swap4Bytes(val) \
 ( (((val) >> 24) & 0x000000FF) | (((val) >>  8) & 0x0000FF00) | \
   (((val) <<  8) & 0x00FF0000) | (((val) << 24) & 0xFF000000) )

// Swap 8 byte, 64 bit values:

#define Swap8Bytes(val) \
 ( (((val) >> 56) & 0x00000000000000FF) | (((val) >> 40) & 0x000000000000FF00) | \
   (((val) >> 24) & 0x0000000000FF0000) | (((val) >>  8) & 0x00000000FF000000) | \
   (((val) <<  8) & 0x000000FF00000000) | (((val) << 24) & 0x0000FF0000000000) | \
   (((val) << 40) & 0x00FF000000000000) | (((val) << 56) & 0xFF00000000000000) )
	
static const char REPRE[] = "LE";

int main(int argc, char **argv) {
	
	printf("Commandline argument count: %d\n", argc);
        printf("Sizeof short[bytes]: %ld, int %ld\n", sizeof(short), sizeof(int));
	unsigned short a = 0x1234;
	unsigned int b = 0x11223344;
	signed short c = -30000;
	signed int d = -30000000;
	size_t b_written = -1;		//store no_bytes_written
 	size_t b_expected = -2;		//expected to be stored



	if (argc != 2) {
		fprintf(stderr, "Usage: %s filename\n", argv[0]);
		exit(ERR_USAGE);
	}

	FILE *f = fopen(argv[1], "w");

	if (f == NULL) {
		fprintf(stderr, "Error (%d) creating file: %s\n", errno, argv[1]);
		exit(ERR_CREATE_FILE);
	}

	//Write information: LE

	const char le_msg[] = "Writing representation: ";
	b_expected = sizeof(le_msg) + sizeof(REPRE);
	b_written = fwrite(le_msg, sizeof(le_msg), 1, f) * sizeof(le_msg) + fwrite(REPRE, sizeof(REPRE), 1, f) * sizeof(REPRE);
	if (b_written == b_expected) {
		printf("Message written to file: %s %s\n", le_msg, REPRE);
	} else {
		fprintf(stderr, "Error writing message %s\nBytes expected: %ld, bytes written: %ld\n", le_msg, b_expected, b_written);
	}

	//Write uShorts
	for (int i = 1; i < NO_INST; i++) {
		b_expected = sizeof(a) * 1;
		printf("Writing unsigned short (%s, size: %ld): %hu (dec) 0x%hx...", REPRE, sizeof(a), a, a);
		b_written = fwrite(&a, sizeof(a), 1, f) * sizeof(a);
		if (b_written == b_expected) {
			printf(" [OK], bytes_written: %ld\n", b_written);
		} else {
			fprintf(stderr, " [Error]\nBytes expected: %ld, bytes written: %ld\n", b_expected, b_written);
			exit(ERR_WRITE);
		}
		a += i;
	}

	//Write uInts
	for (int i = 0; i < NO_INST; i++) {
		b_expected = sizeof(b) * 1;
		printf("Writing unsigned int (%s, size: %ld): %u (dec) 0x%x...", REPRE, sizeof(b), b, b);
		b_written = fwrite(&b, sizeof(b), 1, f) * sizeof(b);
		if (b_written == b_expected) {
			printf(" [OK], bytes_written: %ld\n", b_written);
		} else {
			fprintf(stderr, " [Error]\nBytes expected: %ld, bytes written: %ld\n", b_expected, b_written);
			exit(ERR_WRITE);
		}
		b /= 2;
	}

	//Write sShorts
	for (int i = 0; i < NO_INST; i++) {
		b_expected = sizeof(c) * 1;
		printf("Writing signed short (%s, size: %ld): %hd (dec) 0x%hx...", REPRE, sizeof(c), c, c);
		b_written = fwrite(&c, sizeof(c), 1, f) * sizeof(c);
		if (b_written == b_expected) {
			printf(" [OK], bytes_written: %ld\n", b_written);
		} else {
			fprintf(stderr, " [Error]\nBytes expected: %ld, bytes written: %ld\n", b_expected, b_written);
			exit(ERR_WRITE);
		}
		c *= 2;
	}

	//Write sInts
	for (int i = 0; i < NO_INST; i++) {
		b_expected = sizeof(d) * 1;
		printf("Writing signed int (%s, size: %ld): %d (dec) 0x%x...", REPRE, sizeof(d), d, d);
		b_written = fwrite(&d, sizeof(d), 1, f) * sizeof(d);
		if (b_written == b_expected) {
			printf(" [OK], bytes_written: %ld\n", b_written);
		} else {
			fprintf(stderr, " [Error]\nBytes expected: %ld, bytes written: %ld\n", b_expected, b_written);
			exit(ERR_WRITE);
		}
		d *= 0x101;
	}


	return 0;	
}
